from typing import List, Optional, Union, Dict, Any
from sqlalchemy.orm import Session

from app.models.album import Album
from app.schemas.album import AlbumCreate, AlbumUpdate


def get_album(db: Session, album_id: int) -> Optional[Album]:
    return db.query(Album).filter(Album.id == album_id).first()


def get_albums(
    db: Session, 
    skip: int = 0, 
    limit: int = 100, 
    artist_id: Optional[int] = None,
    search: Optional[str] = None
) -> List[Album]:
    query = db.query(Album)
    if artist_id:
        query = query.filter(Album.artist_id == artist_id)
    if search:
        query = query.filter(Album.title.ilike(f"%{search}%"))
    return query.offset(skip).limit(limit).all()


def create_album(db: Session, obj_in: AlbumCreate) -> Album:
    db_obj = Album(
        title=obj_in.title,
        release_date=obj_in.release_date,
        cover_image_path=obj_in.cover_image_path,
        description=obj_in.description,
        artist_id=obj_in.artist_id,
    )
    db.add(db_obj)
    db.commit()
    db.refresh(db_obj)
    return db_obj


def update_album(
    db: Session, *, db_obj: Album, obj_in: Union[AlbumUpdate, Dict[str, Any]]
) -> Album:
    if isinstance(obj_in, dict):
        update_data = obj_in
    else:
        update_data = obj_in.dict(exclude_unset=True)
    for field in update_data:
        if hasattr(db_obj, field):
            setattr(db_obj, field, update_data[field])
    db.add(db_obj)
    db.commit()
    db.refresh(db_obj)
    return db_obj


def delete_album(db: Session, *, id: int) -> None:
    album = db.query(Album).get(id)
    if album:
        db.delete(album)
        db.commit()